/****************************************************************************
*                                                                           *
*                  ___  ____  ____   ___  __    ____  ____                  *
*                 / __)(_  _)(  _ \ / __)(  )  ( ___)(  _ \                 *
*                ( (__  _)(_  )   /( (__  )(__  )__)  )(_) )                *
*                 \___)(____)(_)\_) \___)(____)(____)(____/                 *
*                                                                           *
*                                                                           *
*                                                                           *
*   This file is part of Circled                                            *
*                                                                           *
*   Copyright (C) 2011  Fabien LOISON                                       *
*                                                                           *
*   Circled is free software: you can redistribute it and/or modify         *
*   it under the terms of the GNU General Public License as published by    *
*   the Free Software Foundation, either version 3 of the License, or       *
*   (at your option) any later version.                                     *
*                                                                           *
*   This program is distributed in the hope that it will be useful,         *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*   GNU General Public License for more details.                            *
*                                                                           *
*   You should have received a copy of the GNU General Public License       *
*   along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
*                                                                           *
****************************************************************************/


/**
 * This file contains all the class/functions for the Circled game.
 */


#include "_headers.h"


//
// Board
//


/**
 * The constructor of the Board class.
 *
 * Initialize an new board.
 *
 * Args:
 *     terminal -- The Terminal instance.
 *     w -- The width of the board.
 *     h -- The height of the board.
 */
Board::Board(Terminal *terminal, int w, int h)
{
	term = terminal;

	//Check width
	if ( w < 6 || w % 2 != 0)
	{
		width = 6;
	}
	else
	{
		width = w;
	}

	//Check height
	if ( h < 5 || h % 2 == 0)
	{
		height = 5;
	}
	else
	{
		height = h;
	}

	//Initialize the board
	int x;
	int y;
	board = new int* [height];
	for (y=0 ; y<height ; y++)
	{
		board[y] = new int [width];
		for (x=0 ; x<width ; x++)
		{
			set_cell(x, y, 0);
		}
	}

	//Put the players on the board
	set_cell(width / 2 - 1, height / 2, CELL_P1);
	set_cell(width / 2, height / 2, CELL_P2);
}

/**
 * Draw the board on the terminal.
 */
void Board::refresh()
{
	//Calculate the board position
	int board_x = (term->get_width() - width*2) / 2 - 1;
	int board_y = (term->get_height() - height) /2;

	//Draw the board shadow
	term->set_bg_color(16);
	term->draw_rectangle(board_x + 2, board_y + 1,
			board_x + width * 2 + 2, board_y + height + 1);

	//Draw the board
	int x;
	int y;
	for (y=0 ; y<height ; y++)
	{
		term->move_xy(board_x, board_y + y);
		for (x=0 ; x<width ; x++)
		{
			switch (get_cell(x, y))
			{
				case CELL_EMPTY:
					if ((x + y) % 2 == 1)
					{
						term->set_bg_color(253);
					}
					else
					{
						term->set_bg_color(255);
					}
					cout << "  " << flush;
					break;
					
				case CELL_P1:
					term->set_fg_color(255);
					term->set_bg_color(25);
					cout << "P1" << flush;
					break;

				case CELL_P2:
					term->set_fg_color(255);
					term->set_bg_color(34);
					cout << "P2" << flush;
					break;

				case CELL_BRICK:
					term->set_bg_color(124);
					cout << "  " << flush;
					break;
					
			}
		}
	}
}

/**
 * Get the value of the given cell.
 *
 * Args:
 *     x, y -- The coordinate of the cell.
 *
 * Returns:
 *     The value of the cell.
 */
int Board::get_cell(int x, int y)
{
	if (x >= 0 && x < width && y >= 0 && y < height)
	{
		return board[y][x];
	}
	else
	{
		return CELL_BRICK;
	}
}

/**
 * Set the value of the given cell.
 *
 * Args:
 *     x, y -- The coordinate of the cell.
 *     value -- The new value of the cell (CELL_EMPTY, CELL_P1,
 *              CELL_P2 or CELL_BRICK).
 */
void Board::set_cell(int x, int y, int value)
{
	board[y][x] = value;
}

/**
 * Check if the cell is free.
 *
 * Returns:
 *     true if the cell is free, false else.
 */
bool Board::cell_is_free(int x, int y)
{
	if (get_cell(x, y) == CELL_EMPTY)
	{
		return true;
	}
	else
	{
		return false;
	}
}

/**
 * Get the board width.
 *
 * Returns:
 *     The board width.
 */
int Board::get_width()
{
	return width;
}

/**
 * Get the board height.
 *
 * Returns:
 *     The board height.
 */
int Board::get_height()
{
	return height;
}


//
// Piece
//


/**
 * The contructor of the Piece class.
 *
 * Args:
 *     x, y -- The coordinates of the piece.
 */
Piece::Piece(int x, int y)
{
	set_x(x);
	set_y(y);
}

/**
 * Get the x position of the piece.
 *
 * Returns:
 *     The x position of the piece.
 */
int Piece::get_x()
{
	return pos_x;
}

/**
 * Set the x position of the piece.
 *
 * Args:
 *     x -- The new x position.
 */
void Piece::set_x(int x)
{
	pos_x = x;
}

/**
 * Get the y position of the piece.
 *
 * Returns:
 *     The y position of the piece.
 */
int Piece::get_y()
{
	return pos_y;
}

/**
 * Set the y position of the piece.
 *
 * Args:
 *     y -- The new y position.
 */
void Piece::set_y(int y)
{
	pos_y = y;
}


//
// Player
//


/**
 * The constructor of the Player class.
 *
 * Args:
 *     x, y -- The initial position of the player on the board.
 *     nick -- The nickname of the player.
 */
Player::Player(int x, int y, std::string nick)
{
	piece = Piece(x, y);
	nickname = nick;
}

/**
 * Check if the piece of the player can move.
 *
 * Args:
 *     board -- The instance of the board.
 *
 * Returns:
 *     true if the piece can move, false else.
 */
bool Player::can_move(Board *board)
{
	int x;
	int y;
	for (y=piece.get_y()-1 ; y<=piece.get_y()+1 ; y++)
	{
		for (x=piece.get_x()-1 ; x<=piece.get_x()+1 ; x++)
		{
			if (board->cell_is_free(x, y))
			{
				return true;
			}
		}
	}
	return false;
}

/**
 * Ask the player for moving its piece.
 *
 * Args:
 *     board -- The instance of the board.
 */
void Player::move_piece(Board *board)
{
	//TODO
}

/**
 * Ask the player for a new brick.
 *
 * Args:
 *     board -- The instance of the board.
 */
void Player::put_brick(Board *board)
{
	//TODO
}

/**
 * Get the player nickname.
 *
 * Returns:
 *     The player nickname.
 */
std::string Player::get_nickname()
{
	return nickname;
}


//
// functions
//


/**
 * Display the logo/copyright screen with fade-in and fade-out effect.
 *
 * Args:
 *     term -- The Terminal instance.
 */
void start_screen(Terminal *term)
{
	term->set_bg_color(16);
	term->draw_rectangle(0, 0, term->get_width(), term->get_height());
	int banner_x = (term->get_width() - 43) / 2;
	int banner_y = (term->get_height() - 4) / 2.5;
	int copyright_x = (term->get_width() - 33) / 2;
	int color;
	for (color=232 ; color<=255 ; color++)
	{
		term->set_fg_color(color);
		term->print_xy(banner_x, banner_y + 0, "  ___  ____  ____   ___  __    ____  ____  ");
		term->print_xy(banner_x, banner_y + 1, " / __)(_  _)(  _ \\ / __)(  )  ( ___)(  _ \\ ");
		term->print_xy(banner_x, banner_y + 2, "( (__  _)(_  )   /( (__  )(__  )__)  )(_) )");
		term->print_xy(banner_x, banner_y + 3, " \\___)(____)(_)\\_) \\___)(____)(____)(____/ ");
		term->print_xy(copyright_x, term->get_height() - 3, "Copyright (C) 2011  Fabien LOISON");
		usleep(100 * 1000);
	}
	usleep(1000 * 1000);
	for (color=255 ; color>=232 ; color--)
	{
		term->set_fg_color(color);
		term->print_xy(banner_x, banner_y + 0, "  ___  ____  ____   ___  __    ____  ____  ");
		term->print_xy(banner_x, banner_y + 1, " / __)(_  _)(  _ \\ / __)(  )  ( ___)(  _ \\ ");
		term->print_xy(banner_x, banner_y + 2, "( (__  _)(_  )   /( (__  )(__  )__)  )(_) )");
		term->print_xy(banner_x, banner_y + 3, " \\___)(____)(_)\\_) \\___)(____)(____)(____/ ");
		term->print_xy(copyright_x, term->get_height() - 3, "Copyright (C) 2011  Fabien LOISON");
		usleep(100 * 1000);
	}
	term->draw_rectangle(0, 0, term->get_width(), term->get_height());
	term->set_fg_color(255);
}


/**
 * Play... \o/
 *
 * Args:
 *     term -- The Terminal instance.
 */
void play(Terminal *term)
{
	term->set_bg_color(232);
	term->draw_rectangle(0, 0, term->get_width(), term->get_height());
	term->set_bg_color(235);
	term->draw_rectangle(0, 0, term->get_width(), 1);
	term->draw_rectangle(0, term->get_height()-1, term->get_width(), term->get_height());

	Board board(term);
	board.refresh();
}
